@@ -30,28 +30,23 @@ Q_OBJECT
 public:
     BookmarksPlugin(QObject *parent, const char *name, const QStringList &args);
 
-    ~BookmarksPlugin();
-    
 private:
     typedef struct S_URLANDNAME{
 		KURL url;
 		QString sender;
     } URLandName;
     typedef QMap<KIO::TransferJob*,URLandName> JobsToURLsMap;
-    
     JobsToURLsMap m_map;
     BookmarksPrefsSettings m_settings;
-    
-    void addKopeteBookmark( KURL url, QString sender );
-    KURL::List* extractURLsFromString( QString text );
+    void addKopeteBookmark( const KURL& url, const QString& sender );
+    KURL::List* extractURLsFromString( const QString& text );
     KBookmarkGroup getKopeteFolder();
-    KBookmarkGroup getFolder( KBookmarkGroup group, QString folder );
-    bool isURLInGroup( KURL url, KBookmarkGroup group );
-    QTextCodec* getPageEncoding( QByteArray data );
+    KBookmarkGroup getFolder( KBookmarkGroup group, const QString& folder );
+    QTextCodec* getPageEncoding( const QByteArray& data );
 public slots:
     void slotBookmarkURLsInMessage(Kopete::Message & msg);
     void slotReloadSettings();
-    
+
 private slots:
     void slotAddKopeteBookmark( KIO::Job *transfer, const QByteArray &data );
 };
