@@ -30,6 +30,7 @@
 #include <kinputdialog.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kpassivepopup.h>
 
 #include <kopeteuiglobal.h>
 #include <kopeteaway.h>
@@ -57,6 +58,7 @@
 #include "tasks/movecontacttask.h"
 #include "tasks/updatecontacttask.h"
 #include "tasks/updatefoldertask.h"
+#include "ui/gwchatsearchdialog.h"
 #include "ui/gwprivacy.h"
 #include "ui/gwprivacydialog.h"
 #include "ui/gwreceiveinvitationdialog.h"
@@ -68,9 +70,7 @@ GroupWiseAccount::GroupWiseAccount( GroupWiseProtocol *parent, const QString& ac
 {
 	Q_UNUSED( name );
 	// Init the myself contact
-	// FIXME: I think we should add a global self metaContact (Olivier)
-	Kopete::MetaContact *metaContact = new Kopete::MetaContact;
-	setMyself( new GroupWiseContact( this, accountId(), metaContact, 0, 0, 0 ) );
+	setMyself( new GroupWiseContact( this, accountId(), Kopete::ContactList::self()->myself(), 0, 0, 0 ) );
 	myself()->setOnlineStatus( GroupWiseProtocol::protocol()->groupwiseOffline );
 
 	// Contact list management
@@ -81,9 +81,11 @@ GroupWiseAccount::GroupWiseAccount( GroupWiseProtocol *parent, const QString& ac
 
 	m_actionAutoReply = new KAction ( i18n( "&Set Auto-Reply..." ), QString::null, 0, this,
 			SLOT( slotSetAutoReply() ), this, "actionSetAutoReply");
+	m_actionJoinChatRoom = new KAction ( i18n( "&Join Channel..." ), QString::null, 0, this,
+										 SLOT( slotJoinChatRoom() ), this, "actionJoinChatRoom");
 	m_actionManagePrivacy = new KAction ( i18n( "&Manage Privacy..." ), QString::null, 0, this,
 			SLOT( slotPrivacy() ), this, "actionPrivacy");
-
+			
 	m_connector = 0;
 	m_QCATLS = 0;
 	m_tlsHandler = 0;
@@ -106,6 +108,7 @@ KActionMenu* GroupWiseAccount::actionMenu()
 	m_actionManagePrivacy->setEnabled( isConnected() );
 	m_actionMenu->insert( m_actionManagePrivacy );
 	m_actionMenu->insert( m_actionAutoReply );
+	m_actionMenu->insert( m_actionJoinChatRoom );
 	/* Used for debugging */
 	/*
 	theActionMenu->insert( new KAction ( "Test rtfize()", QString::null, 0, this,
@@ -115,7 +118,7 @@ KActionMenu* GroupWiseAccount::actionMenu()
 	return m_actionMenu;
 }
 
-const int GroupWiseAccount::port() const
+int GroupWiseAccount::port() const
 {
 	return configGroup()->readNumEntry( "Port" );
 }
@@ -290,7 +293,7 @@ void GroupWiseAccount::performConnectWithPassword( const QString &password )
 	// not implemented: error
 	QObject::connect( m_clientStream, SIGNAL( error(int) ), SLOT( slotCSError(int) ) );
 
-	m_client = new Client( this );
+	m_client = new Client( this, CMSGPRES_GW_6_5 );
 
 	// NB these are prefixed with QObject:: to avoid any chance of a clash with our connect() methods.
 	// we connected successfully
@@ -306,9 +309,9 @@ void GroupWiseAccount::performConnectWithPassword( const QString &password )
 	// contact status changed
 	QObject::connect( m_client, SIGNAL( statusReceived( const QString &, Q_UINT16, const QString & ) ), SLOT( receiveStatus( const QString &, Q_UINT16 , const QString & ) ) );
 	// incoming message
-	QObject::connect( m_client, SIGNAL( messageReceived( const ConferenceEvent & ) ), SLOT( receiveMessage( const ConferenceEvent & ) ) );
+	QObject::connect( m_client, SIGNAL( messageReceived( const ConferenceEvent & ) ), SLOT( handleIncomingMessage( const ConferenceEvent & ) ) );
 	// auto reply to one of our messages because the recipient is away
-	QObject::connect( m_client, SIGNAL( autoReplyReceived( const ConferenceEvent & ) ), SLOT( receiveAutoReply( const ConferenceEvent & ) ) );
+	QObject::connect( m_client, SIGNAL( autoReplyReceived( const ConferenceEvent & ) ), SLOT( handleIncomingMessage( const ConferenceEvent & ) ) );
 
 	QObject::connect( m_client, SIGNAL( ourStatusChanged( GroupWise::Status, const QString &, const QString & ) ), SLOT( changeOurStatus( GroupWise::Status, const QString &, const QString & ) ) );
 	// conference events
@@ -335,6 +338,10 @@ void GroupWiseAccount::performConnectWithPassword( const QString &password )
 	// privacy - contacts can't connect directly to this signal because myself() is initialised before m_client
 	QObject::connect( m_client->privacyManager(), SIGNAL( privacyChanged( const QString &, bool ) ), SIGNAL( privacyChanged( const QString &, bool ) ) );
 
+	// GW7
+	QObject::connect( m_client, SIGNAL( broadcastReceived( const ConferenceEvent & ) ), SLOT( handleIncomingMessage( const ConferenceEvent & ) ) );
+	QObject::connect( m_client, SIGNAL( systemBroadcastReceived( const ConferenceEvent & ) ), SLOT( handleIncomingMessage( const ConferenceEvent & ) ) );
+
 	struct utsname utsBuf;
 	uname (&utsBuf);
 	m_client->setClientName ("Kopete");
@@ -452,11 +459,15 @@ void GroupWiseAccount::sendInvitation( const GroupWise::ConferenceGuid & guid, c
 void GroupWiseAccount::slotLoggedIn()
 {
 	reconcileOfflineChanges();
-	
+	// set local status display
 	myself()->setOnlineStatus( protocol()->groupwiseAvailable );
-	if ( initialStatus() != Kopete::OnlineStatus(Kopete::OnlineStatus::Online) )
+	// set status on server
+	if ( initialStatus() != Kopete::OnlineStatus(Kopete::OnlineStatus::Online) &&
+		( ( GroupWise::Status )initialStatus().internalStatus() != GroupWise::Unknown ) )
+	{
+		kdDebug( GROUPWISE_DEBUG_GLOBAL ) << "Initial status is not online, setting status to " << initialStatus().internalStatus() << endl;
 		m_client->setStatus( ( GroupWise::Status )initialStatus().internalStatus(), m_initialReason, configGroup()->readEntry( "AutoReply" ) );
-	kdDebug( GROUPWISE_DEBUG_GLOBAL ) << "initial status was " << initialStatus().description() << ", initial reason was " << m_initialReason << endl;
+	}
 }
 
 void GroupWiseAccount::reconcileOfflineChanges()
@@ -551,8 +562,7 @@ void GroupWiseAccount::reconcileOfflineChanges()
 	} //end for, now check the next groupwise contact
 	if ( conflicts )
 		// show queuedmessagebox
-		;
-		//KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Sorry, i18n( "A change happened to your GroupWise contact list while you were offline which was impossible to reconcile." ), i18n( "Conflicting changes made offline" ) );
+		KPassivePopup::message( i18n( "Conflicting Changes Made Offline" ), i18n( "A change happened to your GroupWise contact list while you were offline which was impossible to reconcile." ), Kopete::UI::Global::mainWidget() );
 	m_dontSync = false;
 }
 
@@ -640,6 +650,7 @@ void GroupWiseAccount::slotCSDisconnected()
 	kdDebug ( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << "Disconnected from Groupwise server." << endl;
 	myself()->setOnlineStatus( protocol()->groupwiseOffline );
 	setAllContactsStatus( protocol()->groupwiseOffline );
+	client()->close();
 }
 
 void GroupWiseAccount::slotCSConnected()
@@ -761,22 +772,19 @@ void GroupWiseAccount::slotTLSReady( int secLayerCode )
 	m_client->start( server(), port(), accountId(), password().cachedValue() );
 }
 
-void GroupWiseAccount::receiveMessage( const ConferenceEvent & event )
+void GroupWiseAccount::handleIncomingMessage( const ConferenceEvent & message )
 {
-	kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << " got a message in conference: " << event.guid << ",  from: " << event.user << ", message is: " << event.message << endl;
-	handleIncomingMessage( event, false );
-}
+	QString typeName = "UNKNOWN";
+	if ( message.type == ReceiveMessage )
+		typeName = "message";
+	else if ( message.type == ReceiveAutoReply )
+		typeName = "autoreply";
+	else if ( message.type == ReceivedBroadcast )
+		typeName = "broadcast";
+	else if ( message.type == ReceivedSystemBroadcast )
+		typeName = "system broadcast";
 
-void GroupWiseAccount::receiveAutoReply( const ConferenceEvent & event )
-{
-	kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << " got an auto reply in conference: " << event.guid << ",  from: " << event.user << ", message is: " << event.message << endl;
-	handleIncomingMessage( event, true );
-}
-
-void GroupWiseAccount::handleIncomingMessage( const ConferenceEvent & message,
-bool autoReply )
-{
-	kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << message.user << " sent a " << ( autoReply ? "auto-reply" : "message" ) << " to conference: " << message.guid << ", message: " << message.message << endl;
+	kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << " received a " <<  typeName << " from " << message.user << ", to conference: " << message.guid << ", message: " << message.message << endl;
 
 	GroupWiseContact * sender = contactForDN( message.user );
 	if ( !sender )
@@ -789,18 +797,31 @@ bool autoReply )
 
 	// add an auto-reply indicator if needed
 	QString messageMunged = message.message;
-	if ( autoReply )
+	if ( message.type == ReceiveAutoReply )
 	{
-		QString autoReplyPrefix = i18n("Prefix used for automatically generated auto-reply"
+		QString prefix = i18n("Prefix used for automatically generated auto-reply"
 			" messages when the contact is Away, contains contact's name",
 			"Auto reply from %1: " ).arg( sender->metaContact()->displayName() );
-		messageMunged = autoReplyPrefix + message.message;
+		messageMunged = prefix + message.message;
 	}
+	if ( message.type == GroupWise::ReceivedBroadcast )
+	{
+		QString prefix = i18n("Prefix used for broadcast messages",
+			"Broadcast message from %1: " ).arg( sender->metaContact()->displayName() );
+		messageMunged = prefix + message.message;
+	}
+	if ( message.type == GroupWise::ReceivedSystemBroadcast )
+	{
+		QString prefix = i18n("Prefix used for system broadcast messages",
+			"System Broadcast message from %1: " ).arg( sender->metaContact()->displayName() );
+		messageMunged = prefix + message.message;
+	}
+
 	kdDebug(GROUPWISE_DEBUG_GLOBAL) << k_funcinfo << " message before KopeteMessage and appending: " << messageMunged << endl;
 	Kopete::Message * newMessage = 
 			new Kopete::Message( message.timeStamp, sender, contactList, messageMunged,
 								 Kopete::Message::Inbound, 
-								 autoReply ? Kopete::Message::PlainText : Kopete::Message::RichText );
+								 ( message.type == ReceiveAutoReply ) ? Kopete::Message::PlainText : Kopete::Message::RichText );
 	Q_ASSERT( sess );
 	sess->appendMessage( *newMessage );
 	kdDebug(GROUPWISE_DEBUG_GLOBAL) << "message from KopeteMessage: plainbody: " << newMessage->plainBody() << " parsedbody: " << newMessage->parsedBody() << endl;
@@ -928,7 +949,7 @@ void GroupWiseAccount::receiveAccountDetails( const ContactDetails & details )
 		<< ", givenname" << details.givenName
 		<< ", status" << details.status
 		<< endl;
-	if ( details.cn.lower() == accountId().lower() )
+	if ( details.cn.lower() == accountId().lower().section('@', 0, 0) ) // incase user set account ID foo@novell.com
 	{
 		kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << " - got our details in contact list, updating them" << endl;
 		GroupWiseContact * detailsOwner= static_cast<GroupWiseContact *>( myself() );
@@ -993,6 +1014,7 @@ GroupWiseContact * GroupWiseAccount::createTemporaryContact( const QString & dn
 		metaContact->setDisplayName( displayName );
 		c = new GroupWiseContact( this, details.dn, metaContact, 0, 0, 0 );
 		c->updateDetails( details );
+		c->setProperty( Kopete::Global::Properties::self()->nickName(), protocol()->dnToDotted( details.dn ) );
 		Kopete::ContactList::self()->addMetaContact( metaContact );
 		// the contact details probably don't contain status - but we can ask for it
 		if ( details.status == GroupWise::Invalid && isConnected() )
@@ -1100,8 +1122,15 @@ bool GroupWiseAccount::createContact( const QString& contactId, Kopete::MetaCont
 	//
 	// Since ToMetaContact expects synchronous contact creation
 	// we have to create the contact optimistically.
-	new GroupWiseContact( this, contactId, parentContact, 0, 0, 0 );
+	GroupWiseContact * gc = new GroupWiseContact( this, contactId, parentContact, 0, 0, 0 );
+	ContactDetails dt = client()->userDetailsManager()->details( contactId );
+	QString displayAs;
+	if ( dt.fullName.isEmpty() )
+		displayAs = dt.givenName + " " + dt.surname;
+	else
+		displayAs = dt.fullName;
 
+	gc->setNickName( displayAs );
 	// If the CreateContactTask finishes with an error, we have to
 	// delete the contact we just created, in receiveContactCreated :/
 
@@ -1112,10 +1141,8 @@ bool GroupWiseAccount::createContact( const QString& contactId, Kopete::MetaCont
 	}
 	
 	// get the contact's full name to use as the display name of the created contact
-	ContactDetails dt = client()->userDetailsManager()->details( contactId );
-	
 	CreateContactTask * cct = new CreateContactTask( client()->rootTask() );
-	cct->contactFromUserId( contactId, dt.fullName, highestFreeSequence, folders, topLevel );
+	cct->contactFromUserId( contactId, parentContact->displayName(), highestFreeSequence, folders, topLevel );
 	QObject::connect( cct, SIGNAL( finished() ), SLOT( receiveContactCreated() ) );
 	cct->go( true );
 	return true;
@@ -1158,7 +1185,7 @@ void GroupWiseAccount::receiveContactCreated()
 		KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget (), KMessageBox::Error,
 							i18n ("The contact %1 could not be added to the contact list, with error message: %2").
 							arg(cct->userId() ).arg( cct->statusString() ),
-							i18n ("Error adding contact") );
+							i18n ("Error Adding Contact") );
 	}
 }
 
@@ -1202,8 +1229,8 @@ void GroupWiseAccount::receiveContactDeleted( const ContactItem & instance )
 
 void GroupWiseAccount::slotConnectedElsewhere()
 {
-	KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Information,
-				i18n( "The parameter is the user's own account id for this protocol", "You have been disconnected from GroupWise Messenger because you signed in as %1 elsewhere" ).arg( accountId() ) , i18n ("Signed in as %1 Elsewhere").arg( accountId() ) );
+	KPassivePopup::message( i18n ("Signed in as %1 Elsewhere").arg( accountId() ),
+				i18n( "The parameter is the user's own account id for this protocol", "You have been disconnected from GroupWise Messenger because you signed in as %1 elsewhere" ).arg( accountId() ) , Kopete::UI::Global::mainWidget() );
 	disconnect();
 }
 
@@ -1234,24 +1261,20 @@ void GroupWiseAccount::receiveConferenceJoin( const GroupWise::ConferenceGuid &
 	// find each contact and add them to the GWMM, and tell them they are in the conference
 	for ( QValueList<QString>::ConstIterator it = participants.begin(); it != participants.end(); ++it )
 	{
+		//kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << " adding participant " << *it << endl;
 		GroupWiseContact * c = contactForDN( *it );
-		if ( c )
-		{
-			sess->joined( c );
-		}
-		else
-			kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << " couldn't find a contact for participant DN: " << *it << endl;
+		if ( !c )
+			c = createTemporaryContact( *it );
+		sess->joined( c );	
 	}
 	// add each invitee too
 	for ( QValueList<QString>::ConstIterator it = invitees.begin(); it != invitees.end(); ++it )
 	{
+		//kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << " adding invitee " << *it << endl;
 		GroupWiseContact * c = contactForDN( *it );
-		if ( c )
-		{
-			sess->addInvitee( c );
-		}
-		else
-			kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << " couldn't find a contact for invitee DN: " << *it << endl;
+		if ( !c )
+			c = createTemporaryContact( *it );
+		sess->addInvitee( c );
 	}
 	sess->view( true )->raise( false );
 }
@@ -1368,6 +1391,11 @@ void GroupWiseAccount::slotPrivacy()
 	new GroupWisePrivacyDialog( this, Kopete::UI::Global::mainWidget(), "gwprivacydialog" );
 }
 
+void GroupWiseAccount::slotJoinChatRoom()
+{
+	new GroupWiseChatSearchDialog( this, Kopete::UI::Global::mainWidget(), "gwjoinchatdialog" );
+}
+
 bool GroupWiseAccount::isContactBlocked( const QString & dn )
 {
 	if ( isConnected() )
