@@ -2,7 +2,7 @@
     msnswitchboardsocket.cpp - switch board connection socket
 
     Copyright (c) 2002      by Martijn Klingens       <klingens@kde.org>
-    Copyright (c) 2002-2005 by Olivier Goffart        <ogoffart@ kde.org>
+    Copyright (c) 2002-2006 by Olivier Goffart        <ogoffart@ kde.org>
     Kopete    (c) 2002-2005 by the Kopete developers  <kopete-devel@kde.org>
 
     Portions of this code are taken from KMerlin,
@@ -21,6 +21,7 @@
 
 #include "msnswitchboardsocket.h"
 
+#include <stdlib.h>
 #include <time.h>
 #include <cmath>
 
@@ -29,6 +30,8 @@
 #include <qregexp.h>
 #include <qimage.h>
 #include <qtimer.h>
+#include <qfile.h>
+#include <qfileinfo.h>
 
 // kde
 #include <kdebug.h>
@@ -37,10 +40,13 @@
 #include <kaboutdata.h>
 #include <ktempfile.h>
 #include <kconfig.h>
+#include <kmdcodec.h>
+#include <kstandarddirs.h>
+#include <ktempfile.h>
 
 // for the display picture
-#include "msnp2pdisplatcher.h"
 #include <msncontact.h>
+#include "msnnotifysocket.h"
 
 //kopete
 #include "msnaccount.h"
@@ -48,17 +54,26 @@
 #include "kopetemessage.h"
 #include "kopetecontact.h"
 #include "kopeteuiglobal.h"
+#include "private/kopeteemoticons.h"
+//#include "kopeteaccountmanager.h"
+//#include "kopeteprotocol.h"
 
+#include "sha1.h"
 
+#include "dispatcher.h"
+using P2P::Dispatcher;
 
 MSNSwitchBoardSocket::MSNSwitchBoardSocket( MSNAccount *account , QObject *parent )
 : MSNSocket( parent )
 {
 	m_account = account;
-	m_p2p=0l;
 	m_recvIcons=0;
 	m_emoticonTimer=0L;
 	m_chunks=0;
+	m_clientcapsSent=false;
+	m_dispatcher = 0l;
+	m_keepAlive = 0l;
+	m_keepAliveNb=0;
 }
 
 MSNSwitchBoardSocket::~MSNSwitchBoardSocket()
@@ -70,8 +85,6 @@ MSNSwitchBoardSocket::~MSNSwitchBoardSocket()
 	{
 		delete it.data().second;
 	}
-
-
 }
 
 void MSNSwitchBoardSocket::connectToSwitchBoard(QString ID, QString address, QString auth)
@@ -83,8 +96,8 @@ void MSNSwitchBoardSocket::connectToSwitchBoard(QString ID, QString address, QSt
 	QString server = address.left( address.find( ":" ) );
 	uint port = address.right( address.length() - address.findRev( ":" ) - 1 ).toUInt();
 
-	QObject::connect( this, SIGNAL( blockRead( const QString & ) ),
-		this, SLOT(slotReadMessage( const QString & ) ) );
+	QObject::connect( this, SIGNAL( blockRead( const QByteArray & ) ),
+		this, SLOT(slotReadMessage( const QByteArray & ) ) );
 
 	QObject::connect( this, SIGNAL( onlineStatusChanged( MSNSocket::OnlineStatus ) ),
 		this, SLOT( slotOnlineStatusChanged( MSNSocket::OnlineStatus ) ) );
@@ -97,49 +110,69 @@ void MSNSwitchBoardSocket::connectToSwitchBoard(QString ID, QString address, QSt
 
 void MSNSwitchBoardSocket::handleError( uint code, uint id )
 {
+	kdDebug(14140) << k_funcinfo << endl;
+
+	QString msg;
+	MSNSocket::ErrorType type;
+
 	switch( code )
 	{
 		case 208:
 		{
-			QString msg = i18n( "Invalid user:\n"
+			msg = i18n( "Invalid user:\n"
 				"this MSN user does not exist; please check the MSN ID." );
-			KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Error, msg, i18n( "MSN Plugin" ) );
+			type = MSNSocket::ErrorServerError;
+
 			userLeftChat(m_msgHandle , i18n("user never joined"));
 			break;
 		}
 		case 215:
 		{
-			QString msg = i18n( "The user %1 is already in this chat." ).arg( m_msgHandle );
-			KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Error, msg, i18n( "MSN Plugin" ) );
+			msg = i18n( "The user %1 is already in this chat." ).arg( m_msgHandle );
+			type = MSNSocket::ErrorServerError;
+
 			//userLeftChat(m_msgHandle , i18n("user was twice in this chat") ); //(the user shouln't join there
 			break;
 		}
 		case 216:
 		{
-			QString msg = i18n( "The user %1 is online but has blocked you:\nyou can not talk to this user." ).arg( m_msgHandle );
-			KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Information, msg, i18n( "MSN Plugin" ) );
+			msg = i18n( "The user %1 is online but has blocked you:\nyou can not talk to this user." ).arg( m_msgHandle );
+			type = MSNSocket::ErrorInformation;
+
 			userLeftChat(m_msgHandle, i18n("user blocked you"));
 			break;
 		}
 		case 217:
 		{
 			// TODO: we need to know the nickname instead of the handle.
-			QString msg = i18n( "The user %1 is currently not signed in.\n" "Messages will not be delivered." ).arg( m_msgHandle );
-			KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Error, msg, i18n( "MSN Plugin" ) );
+			msg = i18n( "The user %1 is currently not signed in.\n" "Messages will not be delivered." ).arg( m_msgHandle );
+			type = MSNSocket::ErrorServerError;
+
 			userLeftChat(m_msgHandle, i18n("user disconnected"));
 			break;
 		}
 		case 713:
 		{
 			QString msg = i18n( "You are trying to invite too many contacts to this chat at the same time" ).arg( m_msgHandle );
-			KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Information, msg, i18n( "MSN Plugin" ) );
+			type = MSNSocket::ErrorInformation;
+
 			userLeftChat(m_msgHandle, i18n("user blocked you"));
 			break;
 		}
+		case 911:
+		{
+			msg = i18n("Kopete MSN plugin has trouble authenticating with switchboard server.");
+			type = MSNSocket::ErrorServerError;
+
+			break;
+		}
 		default:
 			MSNSocket::handleError( code, id );
 			break;
 	}
+
+	if( !msg.isEmpty() )
+		emit errorMessage( type, msg );
 }
 
 void MSNSwitchBoardSocket::parseCommand( const QString &cmd, uint  id ,
@@ -205,13 +238,15 @@ void MSNSwitchBoardSocket::parseCommand( const QString &cmd, uint  id ,
 	}
 }
 
-void MSNSwitchBoardSocket::slotReadMessage( const QString &msg )
+void MSNSwitchBoardSocket::slotReadMessage( const QByteArray &bytes )
 {
-	QRegExp rx("Content-Type: ([A-Za-z0-9$!*/\\-]*)");
+	QString msg = QString::fromUtf8(bytes, bytes.size());
+
+	QRegExp rx("Content-Type: ([A-Za-z0-9/\\-]*)");
 	rx.search(msg);
 	QString type=rx.cap(1);
 
-	rx=QRegExp("User-Agent: ([A-Za-z0-9.$!*/\\-]*)");
+	rx=QRegExp("User-Agent: ([A-Za-z0-9./\\-]*)");
 	rx.search(msg);
 	QString clientStr=rx.cap(1);
 
@@ -234,7 +269,21 @@ void MSNSwitchBoardSocket::slotReadMessage( const QString &msg )
 		message = message.replace(  "\r\n" ,"" );
 		emit receivedTypingMsg( message.lower(), true );
 	}
-	else if(type=="text/plain"   || type.isEmpty() )
+	else if(type == "text/x-msnmsgr-datacast")
+	{
+		if(msg.contains("ID:"))
+		{
+			QRegExp rx("ID: ([0-9]*)");
+			rx.search(msg);
+			uint dataCastId = rx.cap(1).toUInt();
+			if( dataCastId == 1 )
+			{
+				kdDebug(14140) << k_funcinfo << "Received a nudge !" << endl;
+				emit nudgeReceived(m_msgHandle);
+			}
+		}
+	}
+	else if(type=="text/plain" /*  || type.isEmpty()*/ )
 	{
 		// Some MSN Clients (like CCMSN) don't like to stick to the rules.
 		// In case of CCMSN, it doesn't send what the content type is when
@@ -255,7 +304,7 @@ void MSNSwitchBoardSocket::slotReadMessage( const QString &msg )
 			fontInfo =rx.cap(1);
 
 			color = parseFontAttr(fontInfo, "CO");
-			
+
 			// FIXME: this is so BAAAAAAAAAAAAD :(
 			if (!color.isEmpty() && color.toInt(0,16)!=0)
 			{
@@ -342,11 +391,11 @@ void MSNSwitchBoardSocket::slotReadMessage( const QString &msg )
 		if(chunk != 0 && !m_msgQueue.isEmpty())
 		{
 			QString msg=m_msgQueue.last().plainBody();
-			m_msgQueue.pop_back(); //removes the last item
-			kmsg.setBody( msg+ message, Kopete::Message::PlainText );
+    		m_msgQueue.pop_back(); //removes the last item
+    		kmsg.setBody( msg+ message, Kopete::Message::PlainText );
 		}
 
-		if(chunk == 0 )
+    	if(chunk == 0 )
 			m_chunks=chunks;
 		else if(chunk+1 >=  m_chunks)
 			m_chunks=0;
@@ -367,11 +416,12 @@ void MSNSwitchBoardSocket::slotReadMessage( const QString &msg )
 			emit msgReceived( parseCustomEmoticons( kmsg ) );
 
 	}
-	else if( type== "text/x-mms-emoticon" )
+	else if( type== "text/x-mms-emoticon" || type== "text/x-mms-animemoticon")
 	{
+		// TODO remove Displatcher.
 		KConfig *config = KGlobal::config();
 		config->setGroup( "MSN" );
-		if ( config->readBoolEntry( "useCustomEmoticons", false ) )
+		if ( config->readBoolEntry( "useCustomEmoticons", true ) )
 		{
 			QRegExp rx("([^\\s]*)[\\s]*(<msnobj [^>]*>)");
 			rx.setMinimal(true);
@@ -389,18 +439,9 @@ void MSNSwitchBoardSocket::slotReadMessage( const QString &msg )
 					if(!c)
 						return;
 
-					if(!m_p2p)
-					{
-						m_p2p=new MSNP2PDisplatcher(this , "msnp2p protocol" );
-						QObject::connect( this, SIGNAL( blockRead( const QByteArray & ) ),    m_p2p, SLOT(slotReadMessage( const QByteArray & ) ) );
-						QObject::connect( m_p2p, SIGNAL( sendCommand( const QString &, const QString &, bool , const QByteArray & , bool ) )  ,
-								this , SLOT(sendCommand( const QString &, const QString &, bool , const QByteArray & , bool )));
-						QObject::connect( m_p2p, SIGNAL( fileReceived( KTempFile *, const QString& ) ) , this , SLOT(slotEmoticonReceived( KTempFile *, const QString& ) ) ) ;
-					}
-
 					// we are receiving emoticons, so delay message display until received signal
 					m_recvIcons++;
-					m_p2p->requestDisplayPicture( m_myHandle, m_msgHandle, msnobj );
+					PeerDispatcher()->requestDisplayIcon(m_msgHandle, msnobj);
 				}
 				pos=rx.search(msg, pos+rx.matchedLength());
 			}
@@ -408,13 +449,107 @@ void MSNSwitchBoardSocket::slotReadMessage( const QString &msg )
 	}
 	else if( type== "application/x-msnmsgrp2p" )
 	{
-		if(!m_p2p)
+		PeerDispatcher()->slotReadMessage(m_msgHandle, bytes);
+	}
+	else if( type == "text/x-clientcaps" )
+	{
+		rx=QRegExp("Client-Name: ([A-Za-z0-9.$!*/% \\-]*)");
+		rx.search(msg);
+		clientStr=unescape( rx.cap(1) );
+
+		if( !clientStr.isNull() && !m_msgHandle.isNull())
 		{
-			m_p2p=new MSNP2PDisplatcher(this , "msnp2p protocol" );
-			QObject::connect( this, SIGNAL( blockRead( const QByteArray & ) ),    m_p2p, SLOT(slotReadMessage( const QByteArray & ) ) );
-			QObject::connect( m_p2p, SIGNAL( sendCommand( const QString &, const QString &, bool , const QByteArray & , bool ) )  ,
-				this , SLOT(sendCommand( const QString &, const QString &, bool , const QByteArray & , bool )));
-			QObject::connect( m_p2p, SIGNAL( fileReceived( KTempFile *, const QString& ) ) , this , SLOT(slotEmoticonReceived( KTempFile *, const QString& ) ) ) ;
+			Kopete::Contact *c=m_account->contacts()[m_msgHandle];
+			if(c)
+				c->setProperty(  MSNProtocol::protocol()->propClient , clientStr );
+		}
+
+		if(!m_clientcapsSent)
+		{
+			KConfig *config = KGlobal::config();
+			config->setGroup( "MSN" );
+
+			QString JabberID;
+			if(config->readBoolEntry("SendJabber", true))
+				JabberID=config->readEntry("JabberAccount");
+
+			if(!JabberID.isEmpty())
+				JabberID="JabberID: "+JabberID +"\r\n";
+
+			if( config->readBoolEntry("SendClientInfo", true)   ||  !JabberID.isEmpty())
+			{
+
+				QCString message = QString( "MIME-Version: 1.0\r\n"
+						"Content-Type: text/x-clientcaps\r\n"
+						"Client-Name: Kopete/"+escape(kapp->aboutData()->version())+"\r\n"
+						+JabberID+
+						"\r\n" ).utf8();
+
+				QString args = "U";
+				sendCommand( "MSG", args, true, message );
+			}
+			m_clientcapsSent=true;
+		}
+
+
+	}
+	else if(type == "image/gif" || msg.contains("Message-ID:"))
+	{
+		// Incoming inkformatgif.
+		QRegExp regex("Message-ID: \\{([0-9A-F\\-]*)\\}");
+		regex.search(msg);
+		QString messageId = regex.cap(1);
+		regex = QRegExp("Chunks: (\\d+)");
+		regex.search(msg);
+		QString chunks = regex.cap(1);
+		regex = QRegExp("Chunk: (\\d+)");
+		regex.search(msg);
+		QString chunk = regex.cap(1);
+
+		if(!messageId.isNull())
+		{
+			bool valid = true;
+			// Retrieve the nmber of data chunks.
+			Q_UINT32 numberOfChunks = chunks.toUInt(&valid);
+			if(valid && (numberOfChunks > 1))
+			{
+				regex = QRegExp("base64:([0-9a-zA-Z+/=]+)");
+				regex.search(msg);
+				// Retrieve the first chunk of the ink format gif.
+				QString base64 = regex.cap(1);
+				// More chunks are expected, buffer the chunk received.
+				InkMessage inkMessage;
+				inkMessage.chunks = numberOfChunks;
+				inkMessage.data += base64;
+				m_inkMessageBuffer.insert(messageId, inkMessage);
+			}
+		}
+		else
+		{
+			// There is only one chunk of data.
+			regex = QRegExp("base64:([0-9a-zA-Z+/=]*)");
+			regex.search(msg);
+			// Retrieve the base64 encoded ink data.
+			QString data = regex.cap(1);
+			DispatchInkMessage(data);
+		}
+
+		if(!messageId.isNull())
+		{
+			if(m_inkMessageBuffer.contains(messageId))
+			{
+				if(chunks.isNull())
+				{
+					InkMessage inkMessage = m_inkMessageBuffer[messageId];
+					inkMessage.data += msg.section("\r\n\r\n", -1);
+					if(inkMessage.chunks == chunk.toUInt() + 1)
+					{
+						DispatchInkMessage(inkMessage.data);
+						// Remove the ink message from the buffer.
+						m_inkMessageBuffer.remove(messageId);
+					}
+				}
+			}
 		}
 	}
 	else
@@ -423,6 +558,19 @@ void MSNSwitchBoardSocket::slotReadMessage( const QString &msg )
 	}
 }
 
+void MSNSwitchBoardSocket::DispatchInkMessage(const QString& base64String)
+{
+	QByteArray image;
+	// Convert from base64 encoded string to byte array.
+	KCodecs::base64Decode(base64String.utf8() , image);
+	KTempFile *inkImage = new KTempFile(locateLocal( "tmp", "inkformatgif-" ), ".gif");
+	inkImage->setAutoDelete(true);
+	inkImage->file()->writeBlock(image.data(), image.size());
+	inkImage->file()->close();
+
+	slotEmoticonReceived(inkImage , "inkformatgif");
+	inkImage = 0l;
+}
 
 void MSNSwitchBoardSocket::sendTypingMsg( bool isTyping )
 {
@@ -453,6 +601,54 @@ void MSNSwitchBoardSocket::slotInviteContact(const QString &handle)
 	m_msgHandle=handle;
 	sendCommand( "CAL", handle );
 }
+//
+// Send a custum emoticon
+//
+int MSNSwitchBoardSocket::sendCustomEmoticon(const QString &name, const QString &filename)
+{
+	QString picObj;
+
+	//try to find it in the cache.
+	const QMap<QString, QString> objectList = PeerDispatcher()->objectList;
+	for (QMap<QString,QString>::ConstIterator it = objectList.begin(); it != objectList.end(); ++it )
+	{
+		if(it.data() == filename)
+		{
+			picObj=it.key();
+			break;
+		}
+	}
+
+	if(picObj.isNull())
+	{ //if not found in the cache, generate the picture object
+		QFileInfo fi(filename);
+		// open the icon file
+		QFile pictFile(fi.filePath());
+		if (pictFile.open(IO_ReadOnly)) {
+
+			QByteArray ar = pictFile.readAll();
+			pictFile.close();
+
+			QString sha1d = QString(KCodecs::base64Encode(SHA1::hash(ar)));
+			QString size = QString::number( pictFile.size() );
+			QString all = "Creator" + m_account->accountId() +	"Size" + size + "Type3Location" + fi.fileName() + "FriendlyAAA=SHA1D" + sha1d;
+			QString sha1c = QString(KCodecs::base64Encode(SHA1::hashString(all.utf8())));
+			picObj = "<msnobj Creator=\"" + m_account->accountId() + "\" Size=\"" + size  + "\" Type=\"3\" Location=\""+ fi.fileName() + "\" Friendly=\"AAA=\" SHA1D=\""+sha1d+ "\" SHA1C=\""+sha1c+"\"/>";
+
+			PeerDispatcher()->objectList.insert(picObj, filename);
+		}
+		else
+			return 0;
+	}
+
+	QString msg = "MIME-Version: 1.0\r\n"
+				"Content-Type: text/x-mms-emoticon\r\n"
+				"\r\n" +
+				name + "\t" + picObj + "\t\r\n";
+
+	return sendCommand("MSG", "A", true, msg.utf8());
+
+}
 
 // this sends a short message to the server
 int MSNSwitchBoardSocket::sendMsg( const Kopete::Message &msg )
@@ -463,37 +659,60 @@ int MSNSwitchBoardSocket::sendMsg( const Kopete::Message &msg )
 		return -1;
 	}
 
+#if 0   //this is to test webcam
+	if(msg.plainBody().contains("/webcam"))
+	{
+		PeerDispatcher()->startWebcam( m_myHandle , m_msgHandle);
+		return -3;
+	}
+#endif
+
+	KConfig *config = KGlobal::config();
+	config->setGroup( "MSN" );
+	if ( config->readBoolEntry( "exportEmoticons", false ) )
+	{
+		QMap<QString, QStringList> emap = Kopete::Emoticons::self()->emoticonAndPicList();
+
+		// Check the list for any custom emoticons
+		for (QMap<QString, QStringList>::const_iterator itr = emap.begin(); itr != emap.end(); itr++)
+		{
+			for ( QStringList::const_iterator itr2 = itr.data().constBegin(); itr2 != itr.data().constEnd(); ++itr2 )
+			{
+				if ( msg.plainBody().contains( *itr2 ) )
+					sendCustomEmoticon( *itr2, itr.key() );
+			}
+		}
+	}
+
 	if( msg.format() & Kopete::Message::RichText )
 	{
-		QRegExp rx("^\\s*<img src=\"([^>\"]+)\"[^>]*>\\s*$");
-		if(rx.search(msg.escapedBody()) != -1)
+		QRegExp regex("^\\s*<img src=\"([^>\"]+)\"[^>]*>\\s*$");
+		if(regex.search(msg.escapedBody()) != -1)
 		{
-			if(!m_p2p)
-			{
-				m_p2p=new MSNP2PDisplatcher(this , "msnp2p protocol" );
-				QObject::connect( this, SIGNAL( blockRead( const QByteArray & ) ),    m_p2p, SLOT(slotReadMessage( const QByteArray & ) ) );
-				QObject::connect( m_p2p, SIGNAL( sendCommand( const QString &, const QString &, bool , const QByteArray & , bool ) )  ,
-						this , SLOT(sendCommand( const QString &, const QString &, bool , const QByteArray & , bool )));
-				QObject::connect( m_p2p, SIGNAL( fileReceived( KTempFile *, const QString& ) ) , this , SLOT(slotEmoticonReceived( KTempFile *, const QString& ) ) ) ;
-			}
-
-			m_p2p->sendImage(rx.cap(1));
-
+			// FIXME why are we sending the images.. the contact should request them.
+			PeerDispatcher()->sendImage(regex.cap(1), m_msgHandle);
 			return -3;
 		}
 	}
 
 	// User-Agent is not a official flag, but GAIM has it
+	QString UA;
+	if( config->readBoolEntry("SendClientInfo", true) )
+	{
+		UA="User-Agent: Kopete/"+escape(kapp->aboutData()->version())+"\r\n";
+	}
+
 	QString head =
 		"MIME-Version: 1.0\r\n"
 		"Content-Type: text/plain; charset=UTF-8\r\n"
-		"User-Agent: Kopete/"+escape(kapp->aboutData()->version())+"\r\n"
+		+UA+
 		"X-MMS-IM-Format: ";
 
 	if(msg.font() != QFont() )
 	{
 		//It's verry strange that if the font name is bigger than 31 char, the _server_ close the socket and don't deliver the message.
 		//  the real question is why ?   my guess is that MS patched the server because a bug in their client,  but that's just a guess.
+		//      - Olivier   06-2005
 		head += "FN=" + escape( msg.font().family().left(31));
 		head += "; EF=";
 		if(msg.font().bold())
@@ -507,10 +726,10 @@ int MSNSwitchBoardSocket::sendMsg( const Kopete::Message &msg )
 		head += "; ";
 	}
 	else head+="FN=; EF=; ";
-	/**
+	/*
 	 * I don't know what to set by default, so i decided to set nothing.  CF Bug 82734
 	 * (but don't forgeto to add an empty FN= and EF= ,  or webmessenger will break. (CF Bug 102371) )
-	  else head+="FN=MS%20Serif; EF=; ";
+	else head+="FN=MS%20Serif; EF=; ";
 	 */
 
 	// Color support
@@ -540,7 +759,7 @@ int MSNSwitchBoardSocket::sendMsg( const Kopete::Message &msg )
 		//It's possible to made the opposite client join them, as explained in this MS Word document
 		//http://www.bot-depot.com/forums/index.php?act=Attach&type=post&id=35110
 
-		head+="Message-ID: {7B7B34E6-7A8D-44FF-926C-1799156B58"+QString::number( KApplication::random()%10)+QString::number( KApplication::random()%10)+"}\r\n";
+		head+="Message-ID: {7B7B34E6-7A8D-44FF-926C-1799156B58"+QString::number( rand()%10)+QString::number( rand()%10)+"}\r\n";
 		int len_H=head.utf8().length()+ 14;   //14 is the size of "Chunks: x"
 		//this is the size of each part of the message (excluding the header)
 		int futurmessages_size=1400;  //1400 is a common good size
@@ -604,6 +823,16 @@ int MSNSwitchBoardSocket::sendMsg( const Kopete::Message &msg )
 		}
 		return -2;  //the message hasn't been sent.
 	}
+	
+	if(!m_keepAlive)
+	{
+		m_keepAliveNb=20;
+		m_keepAlive=new QTimer(this);
+		QObject::connect(m_keepAlive, SIGNAL(timeout()) , this , SLOT(slotKeepAliveTimer()));
+		m_keepAlive->start(50*1000);
+	}
+
+	
 	return sendCommand( "MSG", "A", true, (head+"\r\n"+message).utf8() );
 }
 
@@ -611,7 +840,7 @@ void MSNSwitchBoardSocket::slotSocketClosed( )
 {
 	for( QStringList::Iterator it = m_chatMembers.begin(); it != m_chatMembers.end(); ++it )
 	{
-		emit userLeft( (*it), i18n("socket closed"));
+		emit userLeft( (*it), i18n("connection closed"));
 	}
 
 	// we have lost the connection, send a message to chatwindow (this will not displayed)
@@ -644,6 +873,14 @@ void MSNSwitchBoardSocket::slotOnlineStatusChanged( MSNSocket::OnlineStatus stat
 			args = m_myHandle + " " + m_auth + " " + m_ID;
 		}
 		sendCommand( command, args );
+		
+		if(!m_keepAlive)
+		{
+			m_keepAliveNb=20;
+			m_keepAlive=new QTimer(this);
+			QObject::connect(m_keepAlive, SIGNAL(timeout()) , this , SLOT(slotKeepAliveTimer()));
+			m_keepAlive->start(50*1000);
+		}
 	}
 }
 
@@ -660,21 +897,10 @@ void MSNSwitchBoardSocket::userLeftChat(const QString& handle , const QString &r
 
 void MSNSwitchBoardSocket::requestDisplayPicture()
 {
-	MSNContact *c=static_cast<MSNContact*>(m_account->contacts()[m_msgHandle]);
-	if(!c)
-		return;
+	MSNContact *contact = static_cast<MSNContact*>(m_account->contacts()[m_msgHandle]);
+	if(!contact) return;
 
-	if(!m_p2p)
-	{
-		m_p2p=new MSNP2PDisplatcher(this , "msnp2p protocol" );
-		QObject::connect( this, SIGNAL( blockRead( const QByteArray & ) ),    m_p2p, SLOT(slotReadMessage( const QByteArray & ) ) );
-		QObject::connect( m_p2p, SIGNAL( sendCommand( const QString &, const QString &, bool , const QByteArray & , bool ) )  ,
-				this , SLOT(sendCommand( const QString &, const QString &, bool , const QByteArray & , bool )));
-		QObject::connect( m_p2p, SIGNAL( fileReceived( KTempFile *, const QString& ) ) , this , SLOT(slotEmoticonReceived( KTempFile *, const QString& ) ) ) ;
-	}
-
-
-	m_p2p->requestDisplayPicture( m_myHandle, m_msgHandle, c->object());
+	PeerDispatcher()->requestDisplayIcon(m_msgHandle, contact->object());
 }
 
 void  MSNSwitchBoardSocket::slotEmoticonReceived( KTempFile *file, const QString &msnObj )
@@ -692,11 +918,11 @@ void  MSNSwitchBoardSocket::slotEmoticonReceived( KTempFile *file, const QString
 		if ( m_recvIcons <= 0 )
 			cleanQueue();
 	}
-	else if(msnObj=="typewrite")
+	else if(msnObj == "inkformatgif")
 	{
 		QString msg=i18n("<img src=\"%1\" alt=\"Typewrited message\" />" ).arg( file->name() );
 
-	kdDebug(14140) << k_funcinfo << file->name()  <<endl;
+		kdDebug(14140) << k_funcinfo << file->name()  <<endl;
 
 		m_typewrited.append(file);
 		m_typewrited.setAutoDelete(true);
@@ -735,6 +961,31 @@ void  MSNSwitchBoardSocket::slotEmoticonReceived( KTempFile *file, const QString
 	}
 }
 
+void MSNSwitchBoardSocket::slotIncomingFileTransfer(const QString& from, const QString& /*fileName*/, Q_INT64 /*fileSize*/)
+{
+	QPtrList<Kopete::Contact> others;
+	others.append( m_account->myself() );
+	QStringList::iterator it2;
+	for( it2 = m_chatMembers.begin(); it2 != m_chatMembers.end(); ++it2 )
+	{
+		if( *it2 != m_msgHandle )
+			others.append( m_account->contacts()[ *it2 ] );
+	}
+
+	if(!m_account->contacts()[m_msgHandle])
+	{
+		//this may happens if the contact has been deleted.
+		kdDebug(14140) << k_funcinfo <<"WARNING: contact is null, adding it" <<endl;
+		if( !m_chatMembers.contains( m_msgHandle ) )
+			m_chatMembers.append( m_msgHandle );
+		emit userJoined( m_msgHandle , m_msgHandle , false);
+	}
+	QString invite = "Incoming file transfer.";
+	Kopete::Message msg =
+		Kopete::Message(m_account->contacts()[from], others, invite, Kopete::Message::Internal, Kopete::Message::PlainText);
+	emit msgReceived(msg);
+}
+
 void MSNSwitchBoardSocket::cleanQueue()
 {
 	if(m_emoticonTimer)
@@ -789,6 +1040,17 @@ Kopete::Message &MSNSwitchBoardSocket::parseCustomEmoticons(Kopete::Message &kms
 	return kmsg;
 }
 
+int MSNSwitchBoardSocket::sendNudge()
+{
+	QCString message = QString( "MIME-Version: 1.0\r\n"
+			"Content-Type: text/x-msnmsgr-datacast\r\n"
+			"\r\n"
+			"ID: 1\r\n"
+			"\r\n\r\n" ).utf8();
+
+	QString args = "U";
+	return sendCommand( "MSG", args, true, message );
+}
 
 
 
@@ -813,6 +1075,67 @@ QString MSNSwitchBoardSocket::parseFontAttr(QString str, QString attr)
 	return tmp;
 }
 
+Dispatcher* MSNSwitchBoardSocket::PeerDispatcher()
+{
+	if(!m_dispatcher)
+	{
+		// Create a new msnslp dispatcher to handle
+		// all peer to peer requests.
+		QStringList ip;
+		if(m_account->notifySocket())
+		{
+			ip << m_account->notifySocket()->localIP();
+			if(m_account->notifySocket()->localIP() != m_account->notifySocket()->getLocalIP())
+				ip << m_account->notifySocket()->getLocalIP();
+		}
+		m_dispatcher = new Dispatcher(this, m_account->accountId(),ip );
+
+// 		QObject::connect(this, SIGNAL(blockRead(const QByteArray&)), m_dispatcher, SLOT(slotReadMessage(const QByteArray&)));
+// 		QObject::connect(m_dispatcher, SIGNAL(sendCommand(const QString&, const QString&, bool, const QByteArray&, bool)), this, SLOT(sendCommand(const QString&, const QString&, bool, const QByteArray&, bool)));
+		QObject::connect(m_dispatcher, SIGNAL(incomingTransfer(const QString&, const QString&, Q_INT64)), this, SLOT(slotIncomingFileTransfer(const QString&, const QString&, Q_INT64)));
+		QObject::connect(m_dispatcher, SIGNAL(displayIconReceived(KTempFile *, const QString&)), this, SLOT(slotEmoticonReceived( KTempFile *, const QString&)));
+		QObject::connect(this, SIGNAL(msgAcknowledgement(unsigned int, bool)), m_dispatcher, SLOT(messageAcknowledged(unsigned int, bool)));
+		m_dispatcher->m_pictureUrl = m_account->pictureUrl();
+	}
+	return m_dispatcher;
+}
+
+void MSNSwitchBoardSocket::slotKeepAliveTimer( )
+{
+	/*
+	This is a workaround against the bug 113425
+	The problem:  the P2P::Webcam class is parent of us, and when we get deleted, it get deleted.
+			the correct solution would be to change that.
+	The second problem: after one minute of inactivity, the official client close the chat socket.
+	the workaround: we simulate the activity by sending small packet each 50 seconds
+	the nice side effect:  the "xxx has closed the chat" is now meaningfull
+	the bad side effect:  some switchboard connection may be maintained for really long time!
+	 */
+	
+	if ( onlineStatus() != Connected || m_chatMembers.empty())
+	{
+		//we are not yet in a chat.
+		//if we send that command now, we may get disconnected.
+		return;
+	}
+
+
+	QCString message = QString( "MIME-Version: 1.0\r\n"
+			"Content-Type: text/x-keepalive\r\n"
+			"\r\n" ).utf8();
+
+	// Length is appended by sendCommand()
+	QString args = "U";
+	sendCommand( "MSG", args, true, message );
+	
+	m_keepAliveNb--;
+	if(m_keepAliveNb <= 0)
+	{
+		m_keepAlive->deleteLater();
+		m_keepAlive=0L;
+	}
+}
+
 #include "msnswitchboardsocket.moc"
 
 // vim: set noet ts=4 sts=4 sw=4:
