@@ -2,14 +2,11 @@
 /*
  *              kPPP: A pppd Front End for the KDE project
  *
- * $Id: opener.cpp,v 1.52.2.1 2001/11/10 22:14:48 porten Exp $
+ * $Id: opener.cpp 212185 2003-03-07 22:11:39Z waba $
  *
  *              Copyright (C) 1997,98 Bernd Johannes Wuebben,
- *		                      Mario Weilguni,
- *                                    Harri Porten
- *
- *
- * This file was contributed by Harri Porten <porten@tu-harburg.de>
+ *		                      Mario Weilguni
+ *              Copyright (C) 1998-2002 Harri Porten <porten@kde.org>
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +21,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this program; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 /* A note to developers:
@@ -37,14 +34,18 @@
  * o be paranoid and think twice about everything you change.
  */
 
-#include "kpppconfig.h"
+#include <config.h>
 
 #if defined(__osf__) || defined(__svr4__)
 #define _POSIX_PII_SOCKET
 extern "C" int sethostname(char *name, int name_len);
-extern "C" int _Psendmsg();
-extern "C" int _Precvmsg();
+#if !defined(__osf__)
+extern "C" int _Psendmsg(int, void*, int);
+extern "C" int _Precvmsg(int, void*, int);
 #endif
+#endif
+
+#include "kpppconfig.h"
 
 #include <sys/types.h>
 #include <sys/uio.h>
@@ -124,7 +125,7 @@ static int pppdExitStatus = -1;
 static int checkForInterface();
 
 // processing will stop at first file that could be opened successfully
-const char *kppp_syslog[] = { "/var/log/syslog.ppp",
+const char * const kppp_syslog[] = { "/var/log/syslog.ppp",
 			      "/var/log/syslog",
 			      "/var/log/messages",
 			      0 };
@@ -143,7 +144,7 @@ void Opener::mainLoop() {
   int len;
   int fd = -1;
   int flags, mode;
-  const char *device, **logFile;
+  const char *device, * const *logFile;
   union AllRequests request;
   struct ResponseHeader response;
   struct msghdr	msg;
@@ -202,10 +203,8 @@ void Opener::mainLoop() {
 
 	device = deviceByIndex(request.lock.deviceNum);
 	MY_ASSERT(strlen(LOCK_DIR)+strlen(device) < MaxPathLen);
-	strncpy(lockfile, LOCK_DIR"/LCK..", MaxPathLen);
-	strncat(lockfile, device + strlen("/dev/"),
-		MaxPathLen - strlen(lockfile));
-	lockfile[MaxPathLen] = '\0';
+	strlcpy(lockfile, LOCK_DIR"/LCK..", MaxPathLen);
+	strlcat(lockfile, strrchr(device, '/') + 1, MaxPathLen );
 	response.status = 0;
 	// TODO:
 	//   struct stat st;
@@ -420,7 +419,7 @@ bool Opener::createAuthFile(Auth method, char *username, char *password) {
 
   // look for username, "username" or 'username'
   // if you modify this RE you have to adapt regexp's size above
-  sprintf(regexp, "^[ \t]*%s[ \t]\\|^[ \t]*[\"\']%s[\"\']",
+  snprintf(regexp, sizeof(regexp), "^[ \t]*%s[ \t]\\|^[ \t]*[\"\']%s[\"\']",
           username,username);
   MY_ASSERT(regcomp(&preg, regexp, 0) == 0);
 
@@ -525,7 +524,7 @@ bool Opener::execpppd(const char *arguments) {
     case 0:
       // let's parse the arguments the user supplied into UNIX suitable form
       // that is a list of pointers each pointing to exactly one word
-      strcpy(buf, arguments);
+      strlcpy(buf, arguments, sizeof(buf));
       parseargs(buf, args);
       // become a session leader and let /dev/ttySx
       // be the controlling terminal.
@@ -569,7 +568,7 @@ bool Opener::execpppd(const char *arguments) {
 }
 
 
-bool Opener::killpppd() {
+bool Opener::killpppd()const {
   if(pppdPid > 0) {
     Debug2("In killpppd(): Sending SIGTERM to %d\n", pppdPid);
     if(kill(pppdPid, SIGTERM) < 0) {
@@ -663,7 +662,7 @@ int checkForInterface()
     if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
         return 1;               /* can't tell */
 
-    strncpy(ifr.ifr_name, "ppp0", sizeof (ifr.ifr_name));
+    strlcpy(ifr.ifr_name, "ppp0", sizeof (ifr.ifr_name));
     ok = ioctl(s, SIOCGIFFLAGS, (caddr_t) &ifr) >= 0;
     close(s);
 
